/*
 * Created on Jan 8, 2005
 * 
 */
package com.cachingapp.caching;

import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.servlet.ServletContextEvent;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.loanapp.LoanAppHelper;
import com.loanapp.listener.LoanAppContextListener;
import com.cachingapp.cache.CachingException;
import com.cachingapp.cache.ICacheKey;
import com.opensymphony.oscache.base.Cache;
import com.opensymphony.oscache.base.CacheEntry;
import com.opensymphony.oscache.base.Config;
import com.opensymphony.oscache.base.NeedsRefreshException;
import com.opensymphony.oscache.base.algorithm.AbstractConcurrentReadCache;
import com.opensymphony.oscache.general.GeneralCacheAdministrator;

/**
 * @author Administrator
 *  
 */
public aspect OSCache extends ObjectCache {
    private GeneralCacheAdministrator cacheAdmin = new GeneralCacheAdministrator();

    private AbstractConcurrentReadCache cacheMap = null;
    
    Cache cache = null;

    private static final Properties cacheProperties = new Config().getProperties();

    private static Log sLog = LogFactory.getLog(OSCache.class);

    pointcut contextInitialized(ServletContextEvent event) : args(event) 
    && (execution(void LoanAppContextListener.contextInitialized(ServletContextEvent)));

    pointcut contextDestroyed(ServletContextEvent event) : args(event) 
    && (execution(void LoanAppContextListener.contextDestroyed(ServletContextEvent)));

    pointcut getInterestRates(String productGroup) : args(productGroup)
    && (execution(Object LoanAppHelper.getInterestRates(String)));

    public OSCache() {
        //
    }
    
    public void initObjectCache() {
        try {
            String algorithmClass = cacheProperties.getProperty("cache.algorithm");
            sLog.debug("algorithmClass : " + algorithmClass);
            cacheMap = (AbstractConcurrentReadCache) Class.forName(algorithmClass).newInstance();
            sLog.debug("cacheMap : " + cacheMap);
            cache = cacheAdmin.getCache();
            sLog.debug("cache="+cache);
        } catch (Exception e) {
            // Handle cache region initialization failure
            sLog.debug("Error in creating OSCache.");
            sLog.error(e);
        }
    }
    
    public void cleanupObjectCache() {
        cache = null;
    }


   // CACHE METHODS
    public List getInterestRates(String productGroup, String regionName) 
			throws CachingException {
        List interestRates = null;

		long t1 = System.currentTimeMillis();
		
		cacheProperties.list(System.out);

        int refreshPeriod = Integer.parseInt(cacheProperties
                .getProperty("cache.refresh.period"));

        try {
            interestRates = (List)cache.getFromCache(productGroup, refreshPeriod);
        } catch (NeedsRefreshException nre) {
            sLog.error(nre);
        }
		
		try {
		    // The object was not found in the cache
		    if (interestRates == null) {
		        sLog.info("Could not find the object in cache for key " + productGroup);
		    } else {
		        sLog.info("Found the object in cache for productGroup " + productGroup);
		    }
		
		    long t2 = System.currentTimeMillis();
		    sLog.info("Time taken to get cached object is : " + (t2 - t1) + " milli-seconds.");
		
		} catch (Exception e) {
		    throw new CachingException("Exception in getCacheObject()", e);
		}
		return interestRates;
	}
    
    /**
     * Stores CacheObjects in database. Clears old items and caches new.
     */
    public void putCacheObject(Object cacheKey, Object cacheObject,
            String regionName) throws CachingException {
        Object cacheObjectKey = null;
        try {
            // since any cached data is no longer valid, we should
            // remove the item from the cache if it's an update.
            cache.flushEntry(String.valueOf(cacheKey));

            // Check if cacheKey is an instance of CacheKey interface
            if (cacheKey instanceof ICacheKey) {
                cacheObjectKey = ((ICacheKey) cacheKey).getCacheKey();
            } else {
                cacheObjectKey = cacheKey;
            }
            // put the new object in the cache
            cache.putInCache(String.valueOf(cacheObjectKey), cacheObject,
                    new String[] { regionName });
        } catch (Exception e) {
            // Handle failure in removing object or putting object to cache.
            e.printStackTrace();
            sLog.error("Error in putting object into cache");
            throw new CachingException("Error in putting object into cache");
        }
    }

    /**
     * Stores CacheObjects in the cache with the specified day/time to expire
     * the cahed object.
     */
    public void putCacheObject(Object cacheKey, Object cacheObject,
            boolean expireSchedule, String regionName) throws CachingException {
        Object cacheObjectKey = null;
        try {
            // since any cached data is no longer valid, we should
            // remove the item from the cache if it's an update.
            cache.flushEntry(String.valueOf(cacheObject));

            // Check if cacheKey is an instance of CacheKey interface
            if (cacheKey instanceof ICacheKey) {
                cacheObjectKey = ((ICacheKey) cacheKey).getCacheKey();
            } else {
                cacheObjectKey = cacheKey;
            }
            // put the new object in the cache
            cache.putInCache(String.valueOf(cacheObjectKey), cacheObject);
            //        cacheList.putScheduleExpiry(regionName);
        } catch (Exception e) {
            // Handle failure in removing object or putting object to cache.
            sLog.info("Error in removing object or putting object to cache");
            throw new CachingException(
                    "Error in removing object or putting object to cache");
        }
    }

    /**
     * Peeks into the cache for a given key without loading the value on demand.
     * If the value is cached, then it will be returned. If the value is not
     * already cached, then null is returned.
     */
    public Object peek(Object cacheKey, String regionName)
            throws CachingException {
        Object cacheObject = null;
        Object cacheObjectKey = null;
        // Check if cacheKey is an instance of CacheKey interface
        if (cacheKey instanceof ICacheKey) {
            cacheObjectKey = ((ICacheKey) cacheKey).getCacheKey();
        } else {
            cacheObjectKey = cacheKey;
        }

        try {
            if (cache.getFromCache((String) cacheObjectKey,
                    CacheEntry.INDEFINITE_EXPIRY) == null) {
                sLog.debug("Object does not exist in " + regionName + " region.");
            } else {
                sLog.debug("Object exists in the cache in " + regionName + " region.");
                cacheObject = cache.getFromCache(String.valueOf(cacheObjectKey),
                        CacheEntry.INDEFINITE_EXPIRY);
            }
        } catch (NeedsRefreshException nre) {
            cache.putInCache((String) cacheObjectKey, null, new String[] { regionName });
        }
        return cacheObject;
    }
    
    public Set getCacheObjectKeys(String fqnName) throws CachingException {
        sLog.debug("getCacheObjectKeys in OSCache called..");
        throw new CachingException("Not implemented.");
    }    

    public void printCacheStatistics() throws CachingException {
    }
}
