/*
 * Created on Jan 8, 2005
 * 
 */
package com.cachingapp.caching;

import java.util.List;
import java.util.Set;

import javax.servlet.ServletContextEvent;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.cachingapp.cache.CachingException;

/**
 * @author Administrator
 *  
 */
public abstract aspect ObjectCache {
    private static Log sLog = LogFactory.getLog(ObjectCache.class);
    protected String DEFAULT_REGION = "DefaultCache";
    
    abstract pointcut contextInitialized(ServletContextEvent event);
    void around(ServletContextEvent event) : contextInitialized(event) {
        initObjectCache();
    }    

    abstract pointcut contextDestroyed(ServletContextEvent event);
    void around(ServletContextEvent event) : contextDestroyed(event) {
        cleanupObjectCache();
    }

    abstract pointcut getInterestRates(String productGroup);
    List around(String productGroup) : getInterestRates(productGroup) {
        
	  long start = System.currentTimeMillis();

	  List rateList = null;
	  try {
	      if (peek(productGroup) != null)
	      {
	          sLog.debug("Data found in cache.");
	          rateList = getInterestRates(productGroup);
	          return rateList;
	      }
	      sLog.debug("Data not found in cache.");
	      rateList = proceed(productGroup);
	      putCacheObject(productGroup, rateList);

	      long stop = System.currentTimeMillis();
	      sLog.debug("Time taken to get interest rates=" + (stop-start) + " ms.");
	  
	      printCacheStatistics();
	  } catch (Exception e) {
	      e.printStackTrace();
	  }
      return rateList;
   }    

    // ABSTRACT CACHE METHODS
    public abstract void initObjectCache();
    
    public abstract void cleanupObjectCache();
   
    public abstract List getInterestRates(String productGroup, String regionName) 
    		throws CachingException;

    public abstract void putCacheObject(Object cacheKey, Object cacheObject,
            String regionName) throws CachingException;

    public abstract Object peek(Object cacheKey, String regionName)
            throws CachingException;
    
    public abstract Set getCacheObjectKeys(String fqnName) throws CachingException;

    public abstract void printCacheStatistics() throws CachingException;
    
    // CACHE METHODS
	/**
	 * Retrieves a cached object. Default to look in the cache.
	 */
	public List getInterestRates(String productGroup) throws CachingException {
		return getInterestRates(productGroup, DEFAULT_REGION);
	}

	/**
	 * Stores CacheObjects in database using default cache region.
	 * Clears old items and caches new.
	 */
	public void putCacheObject(Object cacheKey, Object cacheObject) throws CachingException {
	    putCacheObject(cacheKey, cacheObject, DEFAULT_REGION);
	}

	/**
	 * Peeks into the cache using default cache region
	 * for a given key without loading the value on demand.
	 * If the value is cached, then it will be returned.
	 * If the value is not already cached, then null is returned.
	 */
	public Object peek(Object cacheKey) throws CachingException {
	    return peek(cacheKey, DEFAULT_REGION);
	}

}
