package com.loanapp;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.loanapp.data.InterestRate;

/**
 * Helper class for the loan app methods.
 *
 * @version 1.0
 * @author Srini Penchikala
 * @since 01/22/2005
 */
public class LoanAppHelper {

    private static Log sLog = LogFactory.getLog(LoanAppHelper.class);
    
    private static String jdbcDriver = "org.hsqldb.jdbcDriver";
    private static String jdbcUrl = "jdbc:hsqldb:hsql://localhost/loandb";
    private static String userName = "sa";
    private static String password= "";
	
    public LoanAppHelper() {
        try {
            // Load the HSQL Database Engine JDBC driver
            Class.forName(jdbcDriver);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public List getInterestRates2(String prodGroup) {
        List interestRates = new ArrayList();
        String productType = "Test Product";
		String productGroup = "Test Product Group";
		double rate = 5.5;
		double mtgPoints = 1.25;
		double apr = 5.75;

		InterestRate interestRate = new InterestRate();
		interestRate.setProductType(productType);
		interestRate.setProductGroup(productGroup);
		interestRate.setRate(rate);
		interestRate.setPoints(mtgPoints);
		interestRate.setApr(apr);

		interestRates.add(interestRate);
		
		return interestRates;
    }

    public List getInterestRates(String prodGroup) {

    	List interestRates = new ArrayList();
    	
		Connection conn = null;
		Statement stmt = null;
		ResultSet rs = null;

    	try {
    		conn = DriverManager.getConnection(jdbcUrl, userName, password);
    		
    		sLog.debug("conn: " + conn);
    		
    		stmt = conn.createStatement();

    		String sql = "SELECT * FROM InterestRates WHERE ";
    		sql += "productGroup = '" + prodGroup + "'";
//    		sql += " AND points >= " + points + " AND ";
//    		sql += "points <= " + (points+1);
    		rs = stmt.executeQuery(sql);

    		List rates = new ArrayList();
    		while (rs.next()) {
    			String productType = rs.getString("productType");
    			String productGroup = rs.getString("productGroup");
    			double rate = rs.getDouble("rate");
    			double mtgPoints = rs.getDouble("points");
    			double apr = rs.getDouble("apr");

    			InterestRate interestRate = new InterestRate();
    			interestRate.setProductType(productType);
    			interestRate.setRate(rate);
    			interestRate.setPoints(mtgPoints);
    			interestRate.setApr(apr);

    			interestRates.add(interestRate);
    		}
    	} catch (SQLException e) {
    		e.printStackTrace();
    	} finally {
    	    try {
	    		rs.close();
	    		stmt.close(); 
	    		conn.close();
    	    } catch (Exception e) {
    	        //
    	    }
    	}
    	return interestRates;
    }

    public static void main(String[] args) {

        LoanAppHelper helper = new LoanAppHelper();
        
        List rates = null;
        
        System.out.println("TEST");
        
        rates = helper.getInterestRates("Mortgage");
        sLog.debug(helper.toString(rates));
        
        helper.getInterestRates("Heloc");
        sLog.debug(helper.toString(rates));

    }
    
    public String toString(List rates) {
        StringBuffer sb = new StringBuffer();
        
        Iterator it = rates.iterator();
        while (it.hasNext()) {
            InterestRate rate = (InterestRate)it.next();
            sb.append(rate.toString()+"\n");
        }
        return sb.toString();
    }

}
// End of LoanAppHelper.java
