package com.loanapp.servlet;

import java.io.IOException;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.loanapp.LoanAppHelper;

public class LoanAppServlet extends HttpServlet {

    private static Log sLog = LogFactory.getLog(LoanAppServlet.class);
    private static LoanAppHelper loanAppHelper = new LoanAppHelper();

    //Initialize global variables
    public void init() throws ServletException {
        sLog.debug(getInitParameter("clusterNode") + "- init() method called.");
    }

    //Clean up resources
    public void destroy() {
        sLog.debug(getInitParameter("clusterNode") + "- destroy() method called.");
    }

    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException {
        sLog.debug(getInitParameter("clusterNode") + "- service() method called.");
        String strResponse = "";

		String productGroup = request.getParameter("loanRequest");
		sLog.debug("loanRequest=" + productGroup);
		if (productGroup == null) productGroup = "Mortgage";

	    try {
	        int points = Integer.parseInt(request.getParameter("points"));
			List mortgageRates = loanAppHelper.getInterestRates(productGroup);
			request.setAttribute("InterestRates",mortgageRates);
	    } catch (Exception e) {
	        e.printStackTrace();
	        generateErrorReport(request, response, "ERROR");
	        throw new ServletException(e);
	    }
		printHeaderInfo(request,response);
        strResponse = generateContinuePage(request, response);
        sLog.debug("strResponse : " + strResponse);

        try {
            response.getOutputStream().write(strResponse.getBytes());
        } catch (IOException ioe) {
            sLog.error(ioe);
            throw new ServletException(ioe);
        }
    }

    public String generateContinuePage(HttpServletRequest request,
            HttpServletResponse response) throws ServletException {
        /**
         * generates a java.util.Properties format response. with the following
         * parameters sessionId param_xx - where xx is a counter on the session
         * attribute name value_xx - where xx is a counter on the session
         * attribute value counter - the number of requests
         */

        sLog.debug(getInitParameter("clusterNode") + "- generateContinuePage() method called.");

        java.util.Properties p = new java.util.Properties();
        java.io.ByteArrayOutputStream out = new java.io.ByteArrayOutputStream();
        java.util.Enumeration er = request.getAttributeNames();
        while (er.hasMoreElements()) {
            String name = (String) er.nextElement();
            p.setProperty(name, request.getAttribute(name).toString());
        }
        try {
            p.store(out, "\n");
            return new String(out.toByteArray());
        } catch (Exception x) {
            throw new ServletException(x);
        }
    }

    public void generateErrorReport(HttpServletRequest request,
            HttpServletResponse response, String message)
            throws ServletException {

        sLog.debug(getInitParameter("clusterNode")
                + "- generateErrorReport() method called.");

        // When an error happens print out headers and more
        StringBuffer strError = new StringBuffer("Error request on session:").append(message);
        strError.append("\nHEADERS:");
        java.util.Enumeration e = request.getHeaderNames();
        while (e.hasMoreElements()) {
            String name = (String) e.nextElement();
            strError.append("\n\t").append(name).append("=").append(request.getHeader(name));
        }
        strError.append("\nPARAMETERS:");
        e = request.getParameterNames();
        while (e.hasMoreElements()) {
            String name = (String) e.nextElement();
            strError.append("\n\t").append(name).append("=").append(request.getParameter(name));
        }
        strError.append("\n");
        sLog.debug(strError);
        try {
            String msg = "[" + java.net.InetAddress.getLocalHost() + "]" + message;
            response.sendError(501, msg);
        } catch (Exception x) {
            x.printStackTrace();
        }
    }

    public void printHeaderInfo(HttpServletRequest request,
            HttpServletResponse response) throws ServletException {

        sLog.debug(getInitParameter("clusterNode") + "- printHeaderInfo() method called.");

        // When an error happens print out headers and more
        StringBuffer strHeader = new StringBuffer("Request Header Info:");
        strHeader.append("\nHEADERS:");
        java.util.Enumeration e = request.getHeaderNames();
        while (e.hasMoreElements()) {
            String name = (String) e.nextElement();
            strHeader.append("\n\t").append(name).append("=").append(request.getHeader(name));
        }
        strHeader.append("\nPARAMETERS:");
        e = request.getParameterNames();
        while (e.hasMoreElements()) {
            String name = (String) e.nextElement();
            strHeader.append("\n\t").append(name).append("=").append(request.getParameter(name));
        }
        strHeader.append("\n");
        sLog.debug(strHeader.toString());
    }
}
